#include "dict.h"

#include <KCodecs>
#include <KLocale>
#include <QTcpSocket>
#include <QTimer>
#include <QRegExp>


const char *Dict::DefaultHost = "dict.org";
const int Dict::DefaultPort = 2628;
const char *Dict::DefaultDatabase = "!";
const char *Dict::DefaultStrategy = ".";
const char *Dict::AllDatabases = "*";

Dict::Dict(const QString &host, int port)
	: socket(0), state(Disconnected), interauth(false), keepconnect(true)
{
	dict.host = host;
	dict.port = port;
	setupSocket();
	setupTimers();
}

void Dict::reset()
{
	delete socket;
	state = Disconnected;
	setupSocket();
}

void Dict::abort()
{
	// Qt before 4.5 fails to abort the connection attempt
	(state == Connecting && qVersion() < QString("4.5"))
		? reset() : socket->abort();
}

void Dict::setupSocket()
{
	socket = new QTcpSocket(this);
	connect(socket, SIGNAL(readyRead()), SLOT(read()));
	connect(socket, SIGNAL(stateChanged(QAbstractSocket::SocketState)),
		SLOT(socketState(QAbstractSocket::SocketState)));
	connect(socket, SIGNAL(error(QAbstractSocket::SocketError)),
		SLOT(socketError(QAbstractSocket::SocketError)));
}

void Dict::setupTimers()
{
	setTimeouts();
	connect(&connectTimer, SIGNAL(timeout()), SLOT(connectTimeout()));
	connect(&readTimer, SIGNAL(timeout()), SLOT(readTimeout()));
	connect(&idleTimer, SIGNAL(timeout()), SLOT(idleTimeout()));
}

void Dict::setTimeouts(int connect, int read, int idle)
{
	connectTimer.setInterval(qMax(connect, 20) * 1000);
	readTimer.setInterval(qMax(read, 10) * 1000);
	idleTimer.setInterval(qMax(idle, 240) * 1000);
}

void Dict::connectTimeout()
{
	connectTimer.stop();
	abort();
	clear();
	emit errorMessage(i18n("Connect Timeout"), ConnectTimeout);
	emit statusMessage(i18n("Canceled"));
}

void Dict::readTimeout()
{
	readTimer.stop();
	abort();
	clear();
	emit errorMessage(i18n("Read Timeout"), ReadTimeout);
	emit statusMessage(i18n("Canceled"));
}

void Dict::idleTimeout()
{
	send(new DictRequest(DictRequest::Quit, "quit"));
}

void Dict::open()
{
	socket->connectToHost(dict.host, dict.port);
	connectTimer.start();
}

void Dict::setHost(const QString &host, int port)
{
	if (dict.host != host || dict.port != port) {
		dict.host = host;
		dict.port = port;
		if (state != Disconnected)
			abort();
		setUser("", "");
		dict.clear();
		clear();
	}
}

void Dict::read()
{
	readTimer.stop();
	int bytes = socket->bytesAvailable();
	if (!bytes) return;

	if (state < Reading) state = Reading;
	emit statusMessage(i18n("Receiving"));
	QByteArray buf = socket->read(bytes);

	// convert from utf only at newline
	// since a multibyte character can get split across packets
	int end = buf.lastIndexOf('\n');
	read(QString::fromUtf8(extra + buf.left(end)).remove('\r'));
	extra = buf.mid(end + 1);
}

void Dict::send(const DictRequest *request)
{
	if (state > Connected) {
		state = Writing;
		socket->write(request->command.toUtf8() + '\n');
		readTimer.start();
		idleTimer.start();
		emit statusMessage(i18n("Sending"));
		recvqueue << request;
	} else {
		if (state == Disconnected)
			open();
		sendqueue << request;
	}
}

inline void clearQueue(DictQueue &list)
{
	while (!list.isEmpty())
		delete list.takeFirst();
}

inline void removeFromQueue(DictQueue &list)
{
	if (!list.isEmpty())
		delete list.takeFirst();
}

void Dict::clear()
{
	clearQueue(sendqueue);
	clearQueue(recvqueue);
}

void Dict::cancel()
{
	connectTimer.stop();
	readTimer.stop();
	clear();

	if (keepconnect && state > Connected) {
		state = Canceling;
		idleTimer.start();
	} else if (state != Disconnected)
		abort();

	emit statusMessage(i18n("Canceled"));
}

void Dict::status()
{
	send(new DictRequest(DictRequest::Status, "status"));
}

void Dict::define(const QString &query, const QString &database)
{
	Q_ASSERT(!database.isEmpty());

	QString command = QString("define %1 \"%2\"")
		.arg(database, QString(query).remove('\"'));
	send(new DictRequest(DictRequest::Define, command));
}

void Dict::match(const QString &query, const QString &database, const QString &strategy)
{
	Q_ASSERT(!database.isEmpty());
	Q_ASSERT(!strategy.isEmpty());

	QString command = QString("match %1 %2 \"%3\"")
		.arg(database, strategy, QString(query).remove('\"'));
	send(new DictRequest(DictRequest::Match, command));
}

void Dict::info(const QString &query)
{
	DictRequest::RequestCommand type;
	if (query == "server")
		type = DictRequest::Server;
	else if (query == "databases")
		type = DictRequest::Databases;
	else if (query == "strategies")
		type = DictRequest::Strategies;
	else
		type = DictRequest::DbInfo;

	send(new DictRequest(type, QString("show %1").arg(query)));
}

void Dict::setClient(const QString &client)
{
	dict.client = client;
}

void Dict::setUser(const QString &username, const QString &password)
{
	dict.user = username;
	dict.password = password;
}

void Dict::auth(const QString &username, const QString &password)
{
	setUser(username, password);
	if (dict.user.isEmpty())
		return;

	interauth = true;
	(state >= Connected) ? auth() : open();
}

void Dict::auth()
{
	KMD5 context(dict.msgid.toAscii());
	context.update(dict.password.toAscii());

	QString command = QString("auth %1 %2")
		.arg(dict.user, QString(context.hexDigest()));
	send(new DictRequest(DictRequest::Auth, command));
}

void Dict::client()
{
	QString command = QString("client \"%1\"").arg(dict.client);
	send(new DictRequest(DictRequest::Client, command));
}

void Dict::serverIdent(const QString &text)
{
	QRegExp rx("(.*) <(.*)> (<.*>)");
	if (rx.indexIn(text) >= 0) {
		QStringList list = rx.capturedTexts();
		dict.ident = list[1];
		dict.capabilities = list[2].split('.');
		dict.msgid = list[3];
	} else
		dict.clear();
}

inline DictResponse::ResponseCode responseCode(const QString &line)
{
	return (DictResponse::ResponseCode)
		((line.size() >= 3) ? line.left(3).toInt() : 0);
}

inline QString responseMessage(const QString &line)
{
	return (line.size() > 4) ? line.mid(4) : "";
}

inline bool endMark(const QString &line)
{
	return (line == ".");
}

void Dict::read(const QString &text)
{
	if (state == Reading) state = Parsing;

	foreach (QString line, text.split('\n')) {
		if (state == Parsing) {
			if (!line.isEmpty())
				parse(line);
		} else if (state == Appending) {
			if (endMark(line)) {
				if (response.type == DictResponse::Words)
					response.defines << response.text;
				state = Parsing;
			} else
				append(line);
		}
	}

	if (state == Parsing) state = Reading;
}

DictPair wordpair(const QString &line)
{
	QString key = line.section('\"', 2, 2).trimmed();
	QString val = line.section('\"', 3, 3);
	return qMakePair(key, val);
}

DictPair matchpair(const QString &line)
{
	QString key = line.section(' ', 0, 0);
	QString val = line.section('\"', 1, 1);
	return qMakePair(key, val);
}

typedef QHash<DictResponse::ResponseCode, DictRequest::RequestCommand> DictMap;

class DictTable
{
public:
	DictTable();
	const DictMap &map() const { return table; }

private:
	DictMap table;
};

DictTable::DictTable()
{
	table[DictResponse::Definitions] = DictRequest::Define;
	table[DictResponse::Words] = DictRequest::Define;
	table[DictResponse::Matches] = DictRequest::Match;
	table[DictResponse::DatabaseList] = DictRequest::Databases;
	table[DictResponse::StrategyList] = DictRequest::Strategies;
	table[DictResponse::DatabaseInfo] = DictRequest::DbInfo;
	table[DictResponse::ServerInfo] = DictRequest::Server;
	table[DictResponse::AuthOK] = DictRequest::Auth;
	table[DictResponse::StatusMessage] = DictRequest::Status;
	table[DictResponse::Hello] = DictRequest::None;
	table[DictResponse::Bye] = DictRequest::Quit;
}

const DictTable table;
const DictMap &map = table.map();

void Dict::parse(const QString &line)
{
	DictResponse::ResponseCode code = responseCode(line);

	const DictRequest *request;
	if (!recvqueue.isEmpty())
		request = recvqueue.first();
	else // dummy request for Hello response
		request = new DictRequest(DictRequest::None, "");

	if (code != DictResponse::OK) {
		response.type = code;
		response.message = responseMessage(line);
		emit responseReceived(response);
		if (map.contains(code) && map.value(code) != request->type)
			qWarning() << "dict parse: unexpected response" << code
				<< "for request" << request->type;
	}

	switch (code) {
	case DictResponse::Definitions:
		response.defines.clear();
		response.database.clear();
		break;
	case DictResponse::Words:
		pair = wordpair(response.message);
		if (pair.first != response.database && !response.database.isEmpty()) {
			emit responseProcessed(response);
			response.defines.clear();
		}
		response.text.clear();
		response.database = pair.first;
		response.description = (pair.second.isEmpty()) ? pair.first : pair.second;
		state = Appending;
		break;
	case DictResponse::Matches:
		response.matches.clear();
		response.database.clear();
		state = Appending;
		break;
	case DictResponse::OK:
		switch (response.type) {
		case DictResponse::Words:
		case DictResponse::Matches:
			response.done = true;
			break;
		case DictResponse::DatabaseList:
		case DictResponse::StrategyList:
		case DictResponse::DatabaseInfo:
		case DictResponse::ServerInfo:
			break;
		case DictResponse::Hello:
		case DictResponse::Bye:
			removeFromQueue(recvqueue);
			return;
			break;
		default:
			qWarning() << "dict parse: unhandled code" << response.type;
			return;
		}
		done();
		response.done = false;
		break;
	case DictResponse::NoMatch:
		response.text = i18n("No matches found");
		done();
		break;
	case DictResponse::DatabaseList:
	case DictResponse::StrategyList:
	case DictResponse::ServerInfo:
	case DictResponse::DatabaseInfo:
		response.list.clear();
		state = Appending;
		break;
	case DictResponse::Hello:
		serverIdent(response.message);
		if (request->type == DictRequest::None)
			delete request;
		clearQueue(recvqueue);
		if (!dict.client.isEmpty())
			client();
		if (!dict.user.isEmpty())
			auth();
		if (!sendqueue.isEmpty()) {
			foreach (request, sendqueue)
				send(request);
			sendqueue.clear();
		}
		emit identified(!dict.ident.isEmpty());
		break;
	case DictResponse::Bye:
		removeFromQueue(recvqueue);
		break;
	case DictResponse::StatusMessage:
		response.text = i18n("Connected to %1.", dict.host);
		done();
		break;
	case DictResponse::AuthOK:
		emit authenticated(true);
		if (interauth) {
			response.text = i18n("Authenticated to %1.", dict.host);
			interauth = false;
			done();
		} else
			removeFromQueue(recvqueue);
		break;
	case DictResponse::AuthDenied:
		emit authenticated(false);
		if (interauth)
			emit errorMessage(i18n("Authentication failed"), code);
		interauth = false;
		removeFromQueue(recvqueue);
		break;
	case DictResponse::AccessDenied:
		emit errorMessage(i18n("Access denied"), code);
		break;
	case DictResponse::InvalidDatabase:
		{
		int sect = (request->type != DictRequest::DbInfo) ? 1 : 2;
		emit errorMessage(i18n("Invalid database: %1",
			request->command.section(' ', sect, sect)), code);
		}
		removeFromQueue(recvqueue);
		break;
	case DictResponse::InvalidStrategy:
		emit errorMessage(i18n("Invalid strategy: %1",
			request->command.section(' ', 2, 2)), code);
		removeFromQueue(recvqueue);
		break;
	case DictResponse::NoDatabase:
		emit errorMessage(i18n("No databases available"), code);
		break;
	case DictResponse::NoStrategy:
		emit errorMessage(i18n("No strategies available"), code);
		break;
	case DictResponse::ServerUnavailable:
		emit errorMessage(i18n("Server temporarily unavailable"), code);
		break;
	case DictResponse::ServerShutdown:
		emit errorMessage(i18n("Server shutting down"), code);
		break;
	default:
		qWarning() << "dict parse: unhandled code" << code;
	}
}

void Dict::done()
{
	emit responseProcessed(response);
	emit responseText(response.text);
	emit statusMessage(i18n("Done"));
	response.clear();
	removeFromQueue(recvqueue);
}

void Dict::append(const QString &line)
{
	switch (response.type) {
	case DictResponse::Words:
		response.text += line;
		break;
	case DictResponse::Matches:
		if (line.isEmpty()) return;
		pair = matchpair(line);
		if (pair.first != response.database && !response.database.isEmpty()) {
			emit responseProcessed(response);
			response.matches.clear();
		}
		response.database = pair.first;
		if (response.matches.isEmpty() || response.matches.last() != pair.second)
			response.matches.append(pair.second);
		break;
	case DictResponse::DatabaseList:
	case DictResponse::StrategyList:
		if (line.isEmpty()) return;
		pair = matchpair(line);
		if (pair.second.isEmpty())
			pair.second = pair.first;
		response.list.append(pair);
		break;
	case DictResponse::ServerInfo:
	case DictResponse::DatabaseInfo:
		response.text += line;
		break;
	default:
		qWarning() << "dict append: unhandled code" << response.type;
	}
	if (!response.isEmpty()) response.text += '\n';
}

void Dict::socketState(QAbstractSocket::SocketState socketState)
{
	QString message;
	switch (socketState) {
	case QAbstractSocket::UnconnectedState:
		readTimer.stop();
		idleTimer.stop();
		state = Disconnected;
		message = i18n("Disconnected");
		emit connected(false);
		break;
	case QAbstractSocket::HostLookupState:
		state = HostLookup;
		message = i18n("Looking for %1...", dict.host);
		break;
	case QAbstractSocket::ConnectingState:
		state = Connecting;
		message = i18n("Connecting to %1...", dict.host);
		break;
	case QAbstractSocket::ConnectedState:
		connectTimer.stop();
		readTimer.start();
		state = Connected;
		message = i18n("Waiting for %1...", dict.host);
		emit connected(true);
		break;
	case QAbstractSocket::ClosingState:
		message = i18n("Disconnecting");
		break;
	default:
		qWarning() << "dict state: unhandled state" << socketState;
	}
	if (!message.isEmpty())
		emit statusMessage(message);
}

void Dict::socketError(QAbstractSocket::SocketError error)
{
	connectTimer.stop();
	QString message;
	NetworkError type = UnknownError;
	switch (error) {
	case QAbstractSocket::RemoteHostClosedError:
		break;
	case QAbstractSocket::ConnectionRefusedError:
		message = i18n("Connection refused by %1", dict.host);
		type = ConnectionRefused;
		break;
	case QAbstractSocket::HostNotFoundError:
		message = i18n("Host %1 not found", dict.host);
		type = HostNotFound;
		break;
	default:
		message = socket->errorString();
// 		qDebug() << "dict error: unhandled error" << error;
	}
	if (!message.isEmpty())
		emit errorMessage(message, type);
}
