#ifndef DICT_H
#define DICT_H

#include <QPair>
#include <QList>
#include <QStringList>
#include <QTimer>
#include <QByteArray>
#include <QTcpSocket>

typedef QPair<QString, QString> DictPair;
typedef QList<DictPair> DictList;

struct DictRequest {
	enum RequestCommand {
		None,
		Define,
		Match,
		Databases,
		Strategies,
		DbInfo,
		Server,
		OptMime,
		Client,
		Auth,
		Status,
		Help,
		Quit
	};
	DictRequest(RequestCommand req, const QString &cmd)
		: type(req), command(cmd) {}
	RequestCommand type;
	QString command;
};

struct DictResponse {
	enum ResponseCode {
		DatabaseList = 110,
		StrategyList = 111,
		DatabaseInfo = 112,
		HelpMessage = 113,
		ServerInfo = 114,
		Definitions = 150,
		Words = 151,
		Matches = 152,

		StatusMessage = 210,
		Hello = 220,
		Bye = 221,
		AuthOK = 230,
		OK = 250,

		ServerUnavailable = 420,
		ServerShutdown = 421,

		UnrecognizedCommand = 500,
		IllegalParameter = 501,
		UnimplementedCommand = 502,
		UnimplementedParameter = 503,
		AccessDenied = 530,
		AuthDenied = 531,
		UnknownMechanism = 532,
		InvalidDatabase = 550,
		InvalidStrategy = 551,
		NoMatch = 552,
		NoDatabase = 554,
		NoStrategy = 555,
	};
	ResponseCode type;
	QString message;
	QString text;
	QString database;
	QString description;
	QStringList defines;
	QStringList matches;
	DictList list;
	bool done;
	bool isEmpty() { return text.isEmpty(); }
	void clear() { text.clear(); type = ResponseCode(0); }
};

typedef QList<const DictRequest *> DictQueue;

class Dict : public QObject
{
	Q_OBJECT

public:
	static const char *DefaultHost;
	static const int DefaultPort;
	static const char *DefaultDatabase;
	static const char *DefaultStrategy;
	static const char *AllDatabases;

	Dict(const QString &host = DefaultHost, int port = DefaultPort);
	void setHost(const QString &hostname, int port = DefaultPort);
	void setUser(const QString &username, const QString &password);
	void setClient(const QString &client);
	void setTimeouts(int connect = 20, int read = 10, int idle = 300);

	QString host() const { return dict.host; }
	int port() const { return dict.port; }
	QString client() const { return dict.client; }
	QString user() const { return dict.user; }
	QString ident() const { return dict.ident; }
	QStringList capabilities() const { return dict.capabilities; }

	enum NetworkError {
		UnknownError,
		HostNotFound = 600,
		ConnectionRefused,
		ConnectTimeout,
		ReadTimeout,
	};

public Q_SLOTS:
	void define(const QString &query, const QString &database = DefaultDatabase);
	void match(const QString &query, const QString &database = DefaultDatabase, const QString &strategy = DefaultStrategy);
	void info(const QString &query);
	void auth(const QString &username, const QString &password);
	void status();
	void cancel();

private Q_SLOTS:
	void read();
	void socketError(QAbstractSocket::SocketError);
	void socketState(QAbstractSocket::SocketState);
	void connectTimeout();
	void readTimeout();
	void idleTimeout();

Q_SIGNALS:
	void responseReceived(const DictResponse &);
	void responseProcessed(const DictResponse &);
	void responseText(const QString &);
	void connected(bool);
	void authenticated(bool);
	void identified(bool);
	void statusMessage(const QString &);
	void errorMessage(const QString &, int code = 0);

private:
	enum DictState {
		Disconnected,
		HostLookup,
		Connecting,
		Connected,
		Writing,
		Reading,
		Parsing,
		Appending,
		Canceling,
	};
	QTcpSocket *socket;

	struct DictInfo {
		QString host;
		int port;
		QString client;
		QString user, password;
		QString ident, msgid;
		QStringList capabilities;
		void clear() { ident.clear(); msgid.clear(); capabilities.clear(); }
	} dict;

	QTimer connectTimer, readTimer, idleTimer;

	DictState state;
	DictResponse response;
	DictPair pair;
	DictQueue sendqueue;
	DictQueue recvqueue;
	QByteArray extra;
	bool interauth;
	bool keepconnect;

	void setupSocket();
	void setupTimers();
	void reset();
	void abort();
	void clear();

	void open();
	void auth();
	void client();
	void serverIdent(const QString &text);

	void send(const DictRequest *request);
	void read(const QString &text);
	void parse(const QString &text);
	void append(const QString &text);
	void done();
};

#endif
