#include "dictbrowser.h"
#include "dictpage.h"

#include <QScrollBar>
#include <QPalette>


DictBrowser::DictBrowser(QWidget *parent)
	: QTextBrowser(parent), histSize(100), plaintext(false)
{
	setOpenLinks(false);
	page = new DictPage();
	entry.type = 0;
	connect(this, SIGNAL(highlighted(const QString &)),
		SLOT(pointCursor(const QString &)));
	connect(this, SIGNAL(cursorPositionChanged()), SLOT(selection()));
	connect(page, SIGNAL(changed(bool)), SLOT(setPage(bool)));
}

DictBrowser::~DictBrowser()
{
	delete page;
}

void DictBrowser::setPage(bool append)
{
	if (!append)
		setPage(page->title(), page->htmltext());
	else
		appendPage(page->title(), page->htmltext());
}

void DictBrowser::moveTo(int steps)
{
	QList<HistoryEntry> &history = (steps > 0)
		? forwardHistory : backwardHistory;

	if (history.isEmpty())
		return;

	QList<HistoryEntry> &other = (steps > 0)
		? backwardHistory : forwardHistory;

	savePosition();
	if (steps < 0)
		steps = -steps;
	while (steps--) {
		other.prepend(entry);
		entry = history.takeFirst();
	}
	displayPage(entry.page);
	restorePosition();

	emit backwardAvailable(!backwardHistory.isEmpty());
	emit forwardAvailable(!forwardHistory.isEmpty());
	emit historyChanged(entry.title);
}

void DictBrowser::savePosition()
{
	entry.hpos = horizontalScrollBar()->value();
	entry.vpos = verticalScrollBar()->value();
}

void DictBrowser::restorePosition()
{
	horizontalScrollBar()->setValue(entry.hpos);
	verticalScrollBar()->setValue(entry.vpos);
}

void DictBrowser::clearHistory()
{
	backwardHistory.clear();
	forwardHistory.clear();
	emit backwardAvailable(false);
	emit forwardAvailable(false);
}

QStringList DictBrowser::historyList(const QList<HistoryEntry> &history, int len) const
{
	QStringList list;
	len = qMin(history.size(), len);
	for (int i = 0; i < len; ++i)
		list << history[i].title;
	return list;
}

void DictBrowser::setPage(const QString &query, const QString &text)
{
	savePosition();
	displayPage(text);

	if ((entry.title != query || entry.type != page->type())
	&& !entry.title.isEmpty()) {
		backwardHistory.prepend(entry);
		forwardHistory.clear();
	}
	if (backwardHistory.size() > histSize)
		backwardHistory.removeLast();

	entry.title = query;
	entry.page = text;
	entry.type = page->type();

	emit backwardAvailable(!backwardHistory.isEmpty());
	emit forwardAvailable(!forwardHistory.isEmpty());
	emit historyChanged(query);
}

void DictBrowser::appendPage(const QString &query, const QString &text)
{
	savePosition();
	append(text);
	restorePosition();
	entry.page += text;
}

void DictBrowser::displayPage(const QString &text)
{
	if (plaintext) setPlainText(text);
	else setHtml(text);
}

void DictBrowser::plainText(bool toggle)
{
	plaintext = toggle;
	displayPage(entry.page);
}

void DictBrowser::reload()
{
	savePosition();
	displayPage(entry.page);
	restorePosition();
}

QString resource(const QString &resfile);

QString DictBrowser::pageText() const
{
	QString html = resource("page.html")
		.arg(entry.title)
		.arg(document()->defaultStyleSheet())
		.arg(entry.page);
	return html;
}

void DictBrowser::setStyleSheet(const QString &style)
{
	document()->setDefaultStyleSheet(style);
	reload();
}

void DictBrowser::setTextFont(const QFont &font)
{
	document()->setDefaultFont(font);
	reload();
}

void DictBrowser::clearSelection()
{
	QTextCursor cursor = textCursor();
	cursor.clearSelection();
	setTextCursor(cursor);
}

void DictBrowser::busyCursor(bool busy)
{
	viewport()->setCursor((busy) ? Qt::BusyCursor : Qt::ArrowCursor);
	cursor = viewport()->cursor();
}

void DictBrowser::pointCursor(const QString &text)
{
	if (text.isEmpty())
		viewport()->setCursor(cursor);
}

void DictBrowser::focusOutEvent(QFocusEvent *event)
{
	QTextEdit::focusOutEvent(event);
}

void DictBrowser::selection()
{
	if (!textCursor().hasSelection())
		return;

	QPalette palette;
	QColor text = palette.color(QPalette::HighlightedText);
	QColor back = palette.color(QPalette::Highlight);

	if (textBackgroundColor().value()
		&& textBackgroundColor().value() < textColor().value()) {
		palette.setColor(QPalette::HighlightedText, back);
		palette.setColor(QPalette::Highlight, text);
	} else {
		palette.setColor(QPalette::HighlightedText, text);
		palette.setColor(QPalette::Highlight, back);
	}
	setPalette(palette);
}
