#include "dictpage.h"

#include <KLocale>
#include <KStandardDirs>
#include <QFile>
#include <QRegExp>


int DictPage::maxMatches = 250;
int DictPage::maxSuggestions = 12;

DictPage::DictPage(const DictResponse &response)
{
	correct = false;
	set(response);
}

void DictPage::set(const DictResponse &response, bool append)
{
	switch (response.type) {
	case DictResponse::Words:
		html = (reformat && !query.startsWith("00"))
			? formatter.format(response)
			: formatter.words(response.defines);
		if (headings) heading(response.description);
		break;
	case DictResponse::Matches:
		if (correct) {
			if (!append) corrections(true);
			suggestions(response.database, response.matches);
			append = true;
		} else {
			matches(response.matches);
			if (headings) heading(response.database);
		}
		break;
	case DictResponse::NoMatch:
		return;
		break;
	case DictResponse::DatabaseList:
		databases(response.list);
		break;
	case DictResponse::StrategyList:
		strategies(response.list);
		break;
	case DictResponse::DatabaseInfo:
		html = formatter.words(response.text);
		break;
	case DictResponse::ServerInfo:
		server(response.text);
		break;
	case DictResponse::StatusMessage:
		status(response.message);
		break;
	default:
		html = response.text;
	}
	typeCode = response.type;
	emit changed(append);
}

void DictPage::nomatches(const QString &text, bool define)
{
	query = text;
	html = "<div class=\"message\">" + ((define)
		? i18n("No definitions found for <span class=\"hword\">%1</span>.", text)
		: i18n("No matches found for <span class=\"hword\">%1</span>.", text))
		+ "</div>";
	typeCode = DictResponse::NoMatch;
	emit changed();
}

void DictPage::corrections(bool found)
{
	html = "<p>" + ((found) ? i18n("Suggestions:") : i18n("No suggestions."));
	emit changed(true);
}

void DictPage::heading(const QString &text)
{
	html.prepend("<div class=\"title\">" + text + "</div>");
}

void DictPage::server(const QString &text)
{
	html = "<div class=\"info\">";
	html += text;
	html += "</div>";
}

void DictPage::status(const Dict *dict)
{
	html = "<div class=\"status\">";
	html += i18n("Connected to dict server");
	html += "<table>";
	status(i18n("Host"), dict->host());
	status(i18n("Port"), QString::number(dict->port()));
	status(i18n("Identification"), dict->ident());
	status(i18n("Capabilities"), dict->capabilities().join(", "));
	html += "<tr>";
	status(i18n("Client"), dict->client());
	status(i18n("User"), dict->user());
	html += "<tr>";
	html += "</table>";
	html += "</div>";
}

static const char *format = "<tr><td class=\"name\">%1:<td>%2";

void DictPage::status(const QString &name, const QString &desc)
{
	if (!desc.isEmpty())
		html += QString(format).arg(name, desc);
}

void DictPage::status(const QString &message)
{
	html += "<div>";
	QRegExp rx("d/m/c = (\\d+)/(\\d+)/(\\d+)");
	if (rx.indexIn(message) > 0) {
		QStringList list = rx.capturedTexts();
		html += i18n("Connection Status");
		QStringList desc;
		desc << i18n("Defines") << i18n("Matches") << i18n("Comparisons");
		html += "<table>";
		for (int i = 0; i < 3; ++i)
			html += QString(format).arg(desc[i], list[i+1]);
		html += "</table>";
	} else
		html += i18n("Status Message") + "<div>" + message + "</div>";
	html += "</div>";
}

void DictPage::matches(const QStringList &list)
{
	html = "<div class=\"match\">";
	int count = qMin(list.size(), maxMatches);
	for (int i = 0; i < count; ++i)
		html += QString("<a href=\"%1\">%1</a>\n").arg(list[i]);
	html += "</div>";
}

void DictPage::suggestions(const QString &database, const QStringList &list)
{
	html = "<div class=\"list\">";
	if (headings) html += database + ": ";
	html += "<span class=\"suggest\">";
	int count = qMin(list.size(), maxSuggestions);
	for (int i = 0; i < count; ++i)
		html += QString("<a href=\"%1\">%1</a>, ").arg(list[i]);
	html.chop(2);
	html += "</span></div>";
}

void DictPage::databases(const DictList &list)
{
	html = "<h4>"+ i18n("Dictionary Databases") + "</h4>";
	html += "<table>";
	foreach (DictPair pair, list)
		html += QString("<tr><td class=\"name\"><a href=\"about:%1\">%1</a><td>%2").arg(pair.first, pair.second);
	html += "</table>";
}

void DictPage::strategies(const DictList &list)
{
	html = "<h4>" + i18n("Matching Strategies") + "</h4>";
	html += "<table>";
	foreach (DictPair pair, list)
		html += QString("<tr><td class=\"name\">%1<td>%2").arg(pair.first, pair.second);
	html += "</table>";
}

QString resourcePath(const QString &resfile)
{
	if (resfile.startsWith(":")) return resfile;
	QString path = KStandardDirs::locate("appdata", resfile);
	if (path.isEmpty()) path = ":" + resfile;
	return path;
}

QString resource(const QString &resfile)
{
	QFile file(resourcePath(resfile));
	bool open = file.open(QFile::ReadOnly);
	Q_ASSERT(open);
	return QString(file.readAll());
}

void DictPage::about(const QString &text, const QString &version)
{
	setTitle(text);
	html = resource("intro.html").arg(version);
	typeCode = 0;
	emit changed();
}

void DictPage::help(const QString &text)
{
	setTitle(text);
	html = resource("help.html");
	typeCode = 0;
	emit changed();
}

void DictPage::error(const QString &text, int code)
{
	QString title, desc;
	QString network = i18n("If you are unable to connect to any host, check your computer's network connection.");
	QString firewall = i18n("If your computer or network is protected by a firewall, check that you can access the network.");
	QString unavail =  i18n("The server could be temporarily unavailable or too busy. Try again in a few moments.");
	QString typing = i18n("Check the address for typing errors.");
	QString web = i18n("If there is a web server on the same address, it may provide information on how to obtain access.");

	switch (code) {
	case Dict::HostNotFound:
		title = i18n("Unable to find the host.");
		desc = "<li>" + i18n("The host with that name may not exist.");
		desc += " " + typing + "<li>" + network + "<li>" + firewall;
		break;
	case Dict::ConnectionRefused:
		title = i18n("Unable to connect to the server.");
		desc = "<li>" + i18n("The host may not run a dict server.");
		desc += " " + typing + "<li>" + unavail;
		break;
	case Dict::ConnectTimeout:
		title = i18n("Unable to connect to the server.");
		desc = "<li>" + unavail;
		desc += "<li>" + network + "<li>" + firewall;
		break;
	case Dict::ReadTimeout:
		title = i18n("Unable to read from the server");
		desc = "<li>" + unavail;
		desc += "<li>" +  i18n("If you receive frequent timeouts, the connection is too slow. Try a different dict server.");
		break;
	case MalformedURL:
		desc = "<li>" + typing;
		break;
	case UnsupportedScheme:
		title = i18n("The protocol is not supported.");
		desc = "<li>" + i18n("Supported protocols are <b>dict</b> and <b>http</b>.");
		break;
	case FileSystem:
		title = i18n("Unable to save the document.");
		desc = "<li>" + i18n("Check that you have the permission to write the file.");
		break;
	case DictResponse::ServerUnavailable:
		desc = "<li>" + i18n("The server is temporarily unavailable or too busy. Try again in a few moments.");
		break;
	case DictResponse::ServerShutdown:
		desc = "<li>" + i18n("The server is shutting down and it is unable to serve any more requests.");
		break;
	case DictResponse::AccessDenied:
		title = i18n("Your computer is not allowed to connect.");
		desc = "<li>" + web;
		break;
	case DictResponse::AuthDenied:
		desc = "<li>" + i18n("See <a href=\"about:server\">Server Info</a> page for information about the server.");
		desc += "<li>" + web;
		break;
	case UnknownDatabase:
	case DictResponse::InvalidDatabase:
		desc = "<li>" + i18n("See <a href=\"about:databases\">Database Info</a> page for a list of available databases.");
		break;
	case DictResponse::InvalidStrategy:
		desc = "<li>" + i18n("See <a href=\"about:strategies\">Strategy Info</a> page for a list of available strategies.");
		break;
	default:
		qWarning() << "dict page: unhandled error" << code;
	}
	html = resource("error.html").arg(text, title, desc);
	typeCode = code;
	emit changed();
}

void DictPage::warning(const QString &text, Warning type)
{
	QString title, desc;

	switch (type) {
	case LongSelection:
		title = i18n("Long Selection");
		desc = i18n("Selected text is longer than configured limit.");
		break;
	case UrlSelection:
		title = i18n("Url Selected");
		desc = i18n("Selected text is a Url.");
		break;
	case UrlRequested:
		title = i18n("Url Requested");
		desc = i18n("Requested text is a Url.");
		break;
	}
	html = resource("warning.html").arg(title, text, desc);
	typeCode = type;
	emit changed();
}
