#include "dictstyle.h"

#include <KIcon>
#include <KColorButton>
#include <KLocale>
#include <KConfig>
#include <KConfigGroup>
#include <KGlobal>


DictStyleItem::DictStyleItem(const QString &name)
	: QTreeWidgetItem(QStringList(name), QTreeWidgetItem::UserType), type(name)
{}

void DictStyleItem::setColorWidgets()
{
	if (!treeWidget()) return;

	KColorButton *text = new KColorButton(treeWidget());
	treeWidget()->setItemWidget(this, Text, text);
	connect(text, SIGNAL(changed (const QColor &)),
		SLOT(setTextColor(const QColor &)));

	KColorButton *back = new KColorButton(treeWidget());
	treeWidget()->setItemWidget(this, Back, back);
	connect(back, SIGNAL(changed (const QColor &)),
		SLOT(setBackColor(const QColor &)));

	if (type == "Link" || type == "Description") {
		back->setEnabled(false);
		back->setToolTip("Currently disabled, as it doesn't render correctly.");
	}
}

void DictStyleItem::setTextColor(const QColor &color)
{
	setForeground(Context, color);
}

void DictStyleItem::setBackColor(const QColor &color)
{
	setBackground(Context, color);
}

int DictStyleItem::style() const
{
	int style = NormalFont;
	if (bold()) style |= BoldFont;
	if (italic()) style |= ItalicFont;
	if (underline()) style |= UnderlineFont;
	return style;
}

bool DictStyleItem::bold() const
{
	return (font(Context).bold());
}

bool DictStyleItem::italic() const
{
	return (font(Context).italic());
}

bool DictStyleItem::underline() const
{
	return (font(Context).underline());
}

QString DictStyleItem::text() const
{
	return foreground(Context).color().name();
}

QString DictStyleItem::back() const
{
	return background(Context).color().name();
}

void DictStyleItem::setCheckBoxes()
{
	setCheckState(Bold, (bold()) ? Qt::Checked : Qt::Unchecked);
	setCheckState(Italic, (italic()) ? Qt::Checked : Qt::Unchecked);
	setCheckState(Underline, (underline()) ? Qt::Checked : Qt::Unchecked);
}

void DictStyleItem::set(int style, const QString &text, const QString &back)
{
	setFlags(Qt::ItemIsEnabled);

	QFont newfont = font(Context);
	newfont.setBold(style & BoldFont);
	newfont.setItalic(style & ItalicFont);
	newfont.setUnderline(style & UnderlineFont);
	setFont(Context, newfont);
	setCheckBoxes();

	setTextColor(QColor(text));
	setBackColor(QColor(back));
	if (treeWidget()) {
		((KColorButton *)treeWidget()->itemWidget(this, Text))->setColor(QColor(text));
		((KColorButton *)treeWidget()->itemWidget(this, Back))->setColor(QColor(back));
	}
}

void DictStyleItem::setFonts(int column)
{
	QFont newfont = font(Context);
	switch (column) {
	case Bold:
		newfont.setBold(!newfont.bold());
		break;
	case Italic:
		newfont.setItalic(!newfont.italic());
		break;
	case Underline:
		newfont.setUnderline(!newfont.underline());
		break;
	}
	setFont(Context, newfont);
	setCheckBoxes();
}


DictStyleConf::DictStyleConf(QWidget *parent)
	: QTreeWidget(parent)
{
	setRootIsDecorated(false);

	QStringList headers;
	headers << i18n("Context") << "" << "" << ""
		<< i18n("Text") << i18n("Background");
	setHeaderLabels(headers);

	headerItem()->setIcon(DictStyleItem::Bold, KIcon("format-text-bold"));
	headerItem()->setIcon(DictStyleItem::Italic, KIcon("format-text-italic"));
	headerItem()->setIcon(DictStyleItem::Underline, KIcon("format-text-underline"));

	for (int i = DictStyleItem::Bold; i <= DictStyleItem::Underline; ++i)
		resizeColumnToContents(i);

	connect(this, SIGNAL(itemActivated(QTreeWidgetItem *, int)),
		SLOT(setFonts(QTreeWidgetItem *, int)));
}

void DictStyleConf::setFonts(QTreeWidgetItem *item, int column)
{
	((DictStyleItem *)item)->setFonts(column);
}

void DictStyleConf::setStyle(DictStyle *style)
{
	if (!topLevelItemCount())
		foreach (DictStyleItem *item, style->itemList()) {
			addTopLevelItem(item);
			item->setColorWidgets();
		}
	style->readConf();
}


DictStyle::DictStyle()
{
	QStringList type;
	type << i18n("Title") << i18n("Headword") << i18n("Abbreviation")
		<< i18n("Pronunciation") << i18n("Description") << i18n("Link");
	foreach (QString style, type)
		stylelist << new DictStyleItem(style);
	readConf();
}

DictStyle::~DictStyle()
{
	foreach (DictStyleItem *item, stylelist)
		if (!item->treeWidget()) delete item;
}

void DictStyle::setDefaults()
{
	QPalette palette;
	QString text = palette.color(QPalette::Text).name();
	QString back = palette.color(QPalette::Base).name();

	stylelist[Title]->set(DictStyleItem::NormalFont, palette.color(QPalette::HighlightedText).name(), palette.color(QPalette::Highlight).name());
	stylelist[Headword]->set(DictStyleItem::BoldFont, text, back);
	stylelist[Abbreviation]->set(DictStyleItem::ItalicFont, text, back);
	stylelist[Pronunciation]->set(DictStyleItem::NormalFont, palette.color(QPalette::Dark).name(), back);
	stylelist[Description]->set(DictStyleItem::NormalFont, text, back);
	stylelist[Link]->set(DictStyleItem::NormalFont, palette.color(QPalette::Link).name(), back);
}

void DictStyle::readConf()
{
	KConfigGroup styleConf(KGlobal::config(), "Style");

	setDefaults();
	foreach (DictStyleItem *item, stylelist) {
		QStringList style;
		style = styleConf.readEntry(item->context(), style);
		if (style.size() == 3)
			item->set(style[0].toInt(), style[1], style[2]);
	}
}

void DictStyle::writeConf()
{
	KConfigGroup styleConf(KGlobal::config(), "Style");

	foreach (DictStyleItem *item, stylelist) {
		QStringList style;
		style << QString::number(item->style())
			<< item->text() << item->back();
		styleConf.writeEntry(item->context(), style);
	}
}

QString resource(const QString &resfile);

QString DictStyle::stylesheet() const
{
	QPalette palette;
	QString text = palette.color(QPalette::Text).name();
	QString back = palette.color(QPalette::Base).name();

	QStringList style;
	foreach (DictStyleItem *item, stylelist) {
		QString itemstyle;
		if (item->bold()) itemstyle += "font-weight: bold; ";
		if (item->italic()) itemstyle += "font-style: italic; ";
		itemstyle += QString("text-decoration: %1; ")
			.arg((item->underline() ? "underline" : "none"));
		if (item->text() != text)
			itemstyle += QString("color: %1; ").arg(item->text());
		if (item->back() != back)
			itemstyle += QString("background: %1; ").arg(item->back());
		style << itemstyle;
	}
	return resource("dikt.css")
		.arg(style[Title], style[Headword], style[Abbreviation],
		style[Pronunciation], style[Description], style[Link]);
}
