#include "dicttracker.h"
#include "dictpage.h"


DictTracker::DictTracker(Dict *d, DictPage *w, DictPage *m)
	: dict(d), wordPage(w), matchPage(m)
{
	correct = false;
	dictinit = Stopped;
	setDefaultDatabase(0);
	clear();
}

void DictTracker::define(const QString &text)
{
	if (!(dictinit & Started)) init();
	query = text;
	reset();
	defining = true;
	dbcount = database.size();
	emit loading(true);

	if (dbcount)
		foreach (QString dbase, database)
			dict->define(query, dbase);
	else {
		dbcount = 1;
		dict->define(query, defaultDatabase);
	}
	emit progress(0, dbcount);
}

void DictTracker::match(const QString &text, const QString &strategy)
{
	if (!(dictinit & Started)) init();
	query = text;
	matching = true;
	dbcount = database.size();
	QString strat = (!strategy.isEmpty()) ? strategy : this->strategy;
	emit loading(true);

	if (dbcount)
		foreach (QString dbase, database)
			dict->match(query, dbase, strat);
	else {
		dbcount = 1;
		dict->match(query, defaultDatabase, strat);
	}
	emit progress(0, dbcount);
}

void DictTracker::info(const QString &text, const QString &info)
{
	if (!(dictinit & Started)) init();
	query = text;
	emit loading(true);

	if (info == "status")
		dict->status();
	else
		dict->info(info);
	emit progress(0, 1);
}

void DictTracker::cancel()
{
	dict->cancel();
	if (dictinit == Started)
		dictinit = Stopped;
	clear();
}

void DictTracker::clear()
{
	reset();
	defining = matching = correcting = appending = automatching = false;
	emit loading(false);
}

void DictTracker::setDatabase(const QStringList &list)
{
	database = list;
}

void DictTracker::setStrategy(const QString &text)
{
	strategy = (!text.isEmpty()) ? text : Dict::DefaultStrategy;
	if (autoStrategy != Dict::DefaultStrategy)
		autoStrategy = strategy;
}

void DictTracker::setDefaultDatabase(int dbase)
{
	defaultDatabase = (dbase) ? Dict::AllDatabases : Dict::DefaultDatabase;
}

void DictTracker::setAutoStrategy(int strat)
{
	autoStrategy = (strat) ? strategy : Dict::DefaultStrategy;
}

void DictTracker::init()
{
	dictinit = Started;
	dict->info("databases");
	dict->info("strategies");
// 	emit loading(true);
}

void DictTracker::done()
{
	if (automatch && matchPage && !automatching && defining)
		automatches();
	else
		clear();
}

void DictTracker::suggestions()
{
	reset();
	correcting = true;
	match(query, Dict::DefaultStrategy);
}

void DictTracker::automatches()
{
	reset();
	automatching = true;
	appending = false;
	correcting = false;
	match(query, autoStrategy);
}

void DictTracker::response(const DictResponse &response)
{
	DictPage *page = wordPage;

	switch (response.type) {
	case DictResponse::Words:
		emit words();
		break;
	case DictResponse::Matches:
		page->correcting(correcting);
		if (correcting)
			emit corrections();
		else if (matchPage) {
			page = matchPage;
			emit matches(!automatching);
		}
		break;
	case DictResponse::NoMatch:
		++misses;
		emit progress(count(), dbcount);
		if (misses < dbcount) {
			if (count() >= dbcount)
				done();
			return;
		} else if (automatching) {
		// do nothing
		} else if (!correcting) {
			page->nomatches(query, defining);
			if (correct) {
				suggestions();
				return;
			}
		} else
			page->corrections(false);
		break;
	case DictResponse::DatabaseList:
		emit databases(response.list);
		if (!(dictinit & Databases)) {
			dictinit |= Databases;
			return;
		}
		break;
	case DictResponse::StrategyList:
		emit strategies(response.list);
		if (!(dictinit & Strategies)) {
// 			emit loading(false);
			dictinit |= Strategies;
			return;
		}
		break;
	case DictResponse::ServerInfo:
	case DictResponse::DatabaseInfo:
		break;
	case DictResponse::StatusMessage:
		page->status(dict);
		break;
	case DictResponse::AuthOK:
		init();
		break;
	default:
		qWarning() << "dict tracker: unhandled type" << response.type;
	}

	page->setTitle(query);
	page->set(response, appending);
	appending = true;

	if (response.type == DictResponse::Words
	|| response.type == DictResponse::Matches) {
		if (response.done) ++hits;
		emit progress(count(), dbcount);
		if (count() >= dbcount)
			done();
	} else {
		if (response.type != DictResponse::NoMatch)
			emit progress(1, 1);
		done();
	}
}
