#include "dictwidgets.h"

QString elide(const QString &text, int len = 60)
{
	return (text.length() > len) ? (text.left(len) + "...") : text;
}


DictInfo::DictInfo::DictInfo(QObject *parent)
	: KActionMenu(parent)
{
	setDelayed(false);
	connect(menu(), SIGNAL(triggered(QAction *)),
		SLOT(actionTriggered(QAction *)));
}

void DictInfo::addAction(const QString &text, const QString &data)
{
	menu()->addAction(text)->setData(data);
}

QStringList DictInfo::list() const
{
	QStringList list;
	foreach (QAction *action, QAction::menu()->actions())
		list << action->data().toString();
	return list;
}

void DictInfo::actionTriggered(QAction *action)
{
	emit selected(action->data().toString());
}


#include <QComboBox>
#include <QToolBar>

DictSelect::DictSelect(QObject *parent)
	: KSelectAction(parent), combobox(0) {}

QString DictSelect::current() const
{
	return (currentItem() >= 0 && actions().count())
		? currentAction()->data().toString() : 0;
}

void DictSelect::setCurrent(const QString &data)
{
	foreach (QAction *action, actions())
		if (action->data().toString() == data) {
			setCurrentAction(action);
			if (combobox) combobox->setCurrentIndex(currentItem());
		}
}

void DictSelect::setList(const DictList &list, const QString &selected)
{
	clear();
	foreach (DictPair pair, list) {
		addAction(elide(pair.second))->setData(pair.first);
		if (combobox) combobox->addItem(pair.first);
	}
	setCurrent(selected);
	setEnabled(!list.isEmpty());
}

void DictSelect::clear()
{
	KSelectAction::clear();
	if (combobox) combobox->clear();
}

void DictSelect::setEnabled(bool enabled)
{
	KSelectAction::setEnabled(enabled);
	if (combobox) combobox->setEnabled(enabled);
}

void DictSelect::setFocus()
{
	if (combobox) combobox->setFocus();
}

void DictSelect::actionTriggered(QAction *action)
{
	emit triggered(action->data().toString());
	emit triggered(currentItem());
}

QWidget *DictSelect::createWidget(QWidget *parent)
{
	if (!qobject_cast<QToolBar *>(parent))
		return KSelectAction::createWidget(parent);

	combobox = new QComboBox(parent);
	combobox->setMaxVisibleItems(12);
	combobox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

	connect(combobox, SIGNAL(currentIndexChanged(const QString &)),
		SLOT(setCurrent(const QString &)));
	connect(this, SIGNAL(triggered(int)),
		combobox, SLOT(setCurrentIndex(int)));
	connect(combobox, SIGNAL(activated(const QString &)),
		SIGNAL(triggered(const QString &)));

	int index = currentItem();
	foreach (QAction *action, actions())
		combobox->addItem(action->data().toString());
	combobox->setCurrentIndex(index);

	combobox->setEnabled(combobox->count());
	return combobox;
}

void DictSelect::deleteWidget(QWidget *widget)
{
	delete combobox;
	combobox = 0;
}


DictMenu::DictMenu(QObject *parent)
	: DictInfo(parent), menuLen(24) {}

void DictMenu::setList(const DictList &list, const QStringList &selected)
{
	clear();

	QMenu *submenu = menu();
	for (int i = 0; i < list.size(); ++i) {
		if (menuLen > 0 && i && !(i % menuLen) && i != (list.size() - 1)) {
			KActionMenu *more = new KActionMenu(i18n("More Databases"), submenu);
			submenu->addSeparator();
			submenu->addAction(more);
			submenu = more->menu();
		}
		QAction *action = submenu->addAction(elide(list[i].second));
		action->setData(list[i].first);
		action->setCheckable(true);
	}
	setCurrent(selected);
	setEnabled(!list.isEmpty());
}

QStringList DictMenu::listData(const QMenu *submenu) const
{
	QStringList list;
	foreach (QAction *action, submenu->actions())
		if (action->isCheckable())
			list << action->data().toString();
		else if (action->menu())
			list += listData(action->menu());
	return list;
}

void DictMenu::setCurrentData(const QStringList &list, const QMenu *submenu)
{
	foreach (QAction *action, submenu->actions())
		if (action->isCheckable())
			action->setChecked(list.contains(action->data().toString()));
		else if (action->menu())
			setCurrentData(list, action->menu());
}

QStringList DictMenu::currentData(const QMenu *submenu) const
{
	QStringList list;
	foreach (QAction *action, submenu->actions())
		if (action->isChecked())
			list << action->data().toString();
		else if (action->menu())
			list += currentData(action->menu());
	return list;
}

void DictMenu::actionTriggered(QAction *action)
{
	emit selected(current());
}


DictView::DictView(QWidget *parent)
	: QListWidget(parent)
{
	setSelectionMode(QAbstractItemView::MultiSelection);
	connect(this, SIGNAL(itemSelectionChanged()), SLOT(selection()));
}

void DictView::selection()
{
	QStringList list;
	for (int i = 0; i < count(); ++i)
		if (item(i)->isSelected())
			list << item(i)->data(Qt::UserRole).toString();
	emit selected(list);
}

void DictView::setList(const DictList &list, const QStringList &selected)
{
	blockSignals(true);
	clear();
	blockSignals(false);
	foreach (DictPair pair, list)
		addItem(elide(pair.second));
	for (int i = 0; i < count(); ++i)
		item(i)->setData(Qt::UserRole, list[i].first);
	setCurrent(selected);
}

QString DictView::data(int index) const
{
	return item(index)->data(Qt::UserRole).toString();
}

void DictView::setCurrent(const QStringList &list)
{
	blockSignals(true);
	for (int i = 0; i < count(); ++i)
		item(i)->setSelected(list.contains(data(i)));
	blockSignals(false);
}

const QList<QListWidgetItem *> DictView::list() const
{
	QList<QListWidgetItem *> itemlist;
	for (int i = 0; i < count(); ++i)
		itemlist << item(i);
	return itemlist;
}


DictDatabase::DictDatabase(QObject *parent)
	: KActionMenu(parent), dbmenu(0), dbedit(0)
{
	setDelayed(false);
	connect(menu(), SIGNAL(triggered(QAction *)),
		SLOT(actionTriggered(QAction *)));

	group = new QActionGroup(this);
}

void DictDatabase::setMenu(DictMenu *menu, QAction *edit)
{
	dbmenu = menu;
	dbedit = edit;
	connect(dbmenu->menu(), SIGNAL(hovered(QAction *)), SLOT(update()));
}

void DictDatabase::setGroups(const DictGroupList &list)
{
	QString text;
	if (group->checkedAction())
		text = group->checkedAction()->text().remove('&');

	foreach (QAction *action, group->actions())
		group->removeAction(action);
	menu()->clear();

	addAction(i18n("First Match"), QStringList(Dict::DefaultDatabase));
	addAction(i18n("All Databases"), QStringList(Dict::AllDatabases));
	if (dbmenu) KActionMenu::addAction(dbmenu);

	addSeparator();
	foreach (DictGroupPair pair, list)
		addAction(pair.first, pair.second);

	foreach (QAction *action, menu()->actions()) {
		action->setCheckable(true);
		if (!action->isSeparator())
			group->addAction(action);
		if (action->text() == text)
			action->setChecked(true);
	}
	if (!group->checkedAction()) setChecked(0);
	addSeparator();
	if (dbedit) KActionMenu::addAction(dbedit);

	setEnabled(true);
}

void DictDatabase::update()
{
	dbmenu->setChecked(true);
	emit selected(dbmenu->current());
}

void DictDatabase::addAction(const QString &text, const QStringList &data)
{
	menu()->addAction(text)->setData(data);
}

void DictDatabase::actionTriggered(QAction *action)
{
	if (action->actionGroup() == group)
		emit selected(action->data().toStringList());
// 	else if (action != dbedit)
// 		emit selected(dbmenu->current());
}

void DictDatabase::setChecked(int select)
{
	if (select < 0 || select >= group->actions().size())
		select = 0;
	group->actions().at(select)->setChecked(true);
}

int DictDatabase::checked() const
{
	QAction *checked = group->checkedAction();
	return (checked) ? group->actions().indexOf(checked) : 0;
}

QStringList DictDatabase::current() const
{
	QAction *action = group->checkedAction();
	return (action == dbmenu)
		? dbmenu->current() : action->data().toStringList();
}

void DictDatabase::setList()
{
	dbmenu->setChecked(true);
}

void DictDatabase::checkList()
{
	QStringList dblist = dbmenu->list();
	dblist << Dict::DefaultDatabase << Dict::AllDatabases;
	foreach (QAction *action, group->actions()) {
		if (action == dbmenu) continue;
		QStringList list;
		foreach (QString db, action->data().toStringList())
			if (dblist.contains(db)) list << db;
		action->setData(list);
	}
}


DictHistory::DictHistory(DictBrowser *parent, Direction dir)
	: QMenu(parent)
{
	browser = parent;
	direction = dir;
	connect(this, SIGNAL(triggered(QAction *)), SLOT(steps(QAction *)));
	connect(this, SIGNAL(aboutToShow()), SLOT(setMenu()));
}

void DictHistory::steps(QAction *action)
{
	browser->moveTo((direction) ? action->data().toInt() : -action->data().toInt());
}

void DictHistory::setMenu()
{
	setList((direction) ? browser->forwardList() : browser->backwardList());
}

void DictHistory::setList(const QStringList &list)
{
	clear();
	for (int i = 0; i < list.size(); ++i)
		addAction(list.at(i))->setData(i + 1);
}


#include <KColorScheme>
#include <KLocale>
#include <QKeyEvent>

DictSearch::DictSearch(DictBrowser *parent)
	: QWidget(parent), browser(parent), found(false)
{
	setupUi(this);

	toolbutton->setIcon(KIcon("dialog-close"));
	nextButton->setIcon(KIcon("go-down-search"));
	prevButton->setIcon(KIcon("go-up-search"));

	connect(pattern, SIGNAL(textChanged(const QString &)),
		SLOT(search(const QString &)));
	connect(nextButton, SIGNAL(pressed()), SLOT(next()));
	connect(prevButton, SIGNAL(pressed()), SLOT(prev()));

	layout()->setAlignment(Qt::AlignLeft);
	layout()->setContentsMargins(4, 0, 4, 0);
	setFocusProxy(pattern);
	QWidget::hide();
}

void DictSearch::show()
{
	QWidget::show();
	setFocus();
	pattern->selectAll();
}

void DictSearch::hide()
{
	browser->setFocus();
	QWidget::hide();
}

void DictSearch::find(QTextDocument::FindFlags backward)
{
	if (!found) {
		if (!isVisible()) show();
		search(pattern->text());
		return;
	}
	browser->savePosition();
	if (backward && browser->textCursor().atStart())
		browser->moveCursor(QTextCursor::End);

	if (browser->find(pattern->text(), backward))
		message->setText(i18n("Text found"));
	else {
		browser->clearSelection();
		browser->moveCursor(QTextCursor::Start);
		browser->restorePosition();
		message->setText(i18n("No more matches"));
	}
}

void DictSearch::search(const QString &text)
{
	QPalette palette;
	browser->savePosition();
	if (text.isEmpty()) {
		browser->clearSelection();
		browser->restorePosition();
		clear();
		nextButton->setEnabled(false);
		prevButton->setEnabled(false);
		pattern->setPalette(palette);
		return;
	}
	nextButton->setEnabled(true);
	prevButton->setEnabled(true);
	browser->moveCursor(QTextCursor::Start);
	if (found = browser->find(text)) {
//		KColorScheme::adjustBackground(palette, KColorScheme::PositiveBackground);
		pattern->setPalette(palette);
		message->setText(i18n("Text found"));
	} else {
		browser->restorePosition();
		KColorScheme::adjustBackground(palette, KColorScheme::NegativeBackground);
		pattern->setPalette(palette);
		message->setText(i18n("Text not found"));
	}
}

void DictSearch::clear()
{
	if (!isVisible())
		pattern->clear();
	message->clear();
	found = false;
}

void DictSearch::keyPressEvent(QKeyEvent *event)
{
	if (event->key() == Qt::Key_Escape)
		hide();
	else
		browser->keyPressEvent(event);
}


#include <KIconLoader>
#include <QPixmap>

DictStatus::DictStatus(QWidget *parent)
	: QWidget(parent)
{
	picture = new QLabel(this);
	label = new QLabel(this);
	QLayout *layout = new QHBoxLayout(this);
	layout->addWidget(picture);
	layout->addWidget(label);
	layout->setContentsMargins(4, 0, 10, 0);
}

void DictStatus::setStatus(const char *icon, const QString &text, bool enabled)
{
	QPixmap pix = SmallIcon(icon, KIconLoader::SizeSmall,
		(enabled) ? KIconLoader::DefaultState : KIconLoader::DisabledState);
	picture->setPixmap(pix);
	label->setText(text);
}

void DictStatus::setTip(const QString &text)
{
	picture->setToolTip(text);
}


DictProgress::DictProgress(QWidget *parent)
	: QProgressBar(parent)
{
	setMaximumSize(100, 16);
	setFormat("");
	hide();

	timer.setInterval(1000);
	timer.setSingleShot(true);
	connect(&timer, SIGNAL(timeout()), SLOT(hide()));
}

void DictProgress::set(int current, int total)
{
	if (!current) {
		timer.stop();
		setRange(0, total);
		show();
	}
	setValue(current);
	if (current == total)
		timer.start();
}


DictEntry::DictEntry(QWidget *parent)
	: QComboBox(parent)
{
	setEditable(true);
	setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	setCompleter(0);
}
