#include "dictwindow.h"
#include "dictwidgets.h"
#include "dictbrowser.h"
#include "dictpage.h"
#include "dictsettings.h"
#include "dicttracker.h"
#include "dictselection.h"
#include "dictstyle.h"
#include "dictgroup.h"
#include "dict.h"

#include <KLocale>
#include <KAction>
#include <KActionCollection>
#include <KStandardAction>
#include <KToolBarPopupAction>
#include <KStatusBar>
#include <KMenuBar>
#include <KPasswordDialog>
#include <KToolInvocation>
#include <KGlobalSettings>
#include <KProtocolManager>
#include <KHelpMenu>
#include <KComponentData>
#include <KAboutData>

#include <QUrl>
#include <QSplitter>
#include <QStackedWidget>
#include <QFileDialog>
#include <QPrinter>
#include <QPrintDialog>
#include <QPrintPreviewDialog>


DictWindow::DictWindow()
	: KXmlGuiWindow(), settings(0)
{
	entry = new DictEntry(this);
	entry->lineEdit()->setMaxLength(maxQueryLen);

	browser = new DictBrowser(this);
	search = new DictSearch(browser);

	databaseView = new DictView(this);
	matchList = new DictBrowser(this);
	matchList->setHistorySize(10);

	sidebar = new QStackedWidget(this);
	sidebar->addWidget(databaseView);
	sidebar->addWidget(matchList);

	splitter = new QSplitter(this);
	splitter->addWidget(browser);
	splitter->addWidget(sidebar);
	splitter->setStretchFactor(0, 1);

	QWidget *widget = new QWidget(this);
	QVBoxLayout *layout = new QVBoxLayout(widget);
	layout->setContentsMargins(0, 0, 0, 0);
	layout->addWidget(splitter);
	layout->addWidget(search);
	setCentralWidget(widget);

	dict = new Dict();
	page = browser->dictPage();
	tracker = new DictTracker(dict, page);
	selection = new DictSelection();
	style = new DictStyle();

	setupActions();
	setupGUI();
	setupBrowser();
	setupDict();
	setupTracker();
	setupStatusbar();

	show();
	readConfig();
}

DictWindow::~DictWindow()
{
	writeConfig();
	delete tracker;
	delete dict;
	delete style;
	delete selection;
}

void DictWindow::newWindow()
{
	writeConfig();
	(new DictWindow())->startup();
}

void DictWindow::startup(const QString &location)
{
	cancel();
	if (location.isEmpty())
		setHost(dicthost());
	else
		define(location);
}

void DictWindow::readConfig()
{
	conf.read();

	QFont font = conf.font();
	browser->setTextFont(font);
	matchList->setTextFont(font);

	setSidebar(conf.sidebar());
	splitter->setSizes(conf.widths());
	sidebar->setVisible(conf.sidebarVisible());
	sidebar->setCurrentIndex(conf.stack());

	selection->setup(conf.selection());

	serverConfig();
	loadConfig();
}

void DictWindow::serverConfig()
{
	QStringList hosts = conf.server();
	QString host = conf.host();
	if (hosts.isEmpty()) intro();
	else entry->setFocus();

	if (hosts.indexOf(host) < 0)
		hosts << host;
	foreach (QString name, hosts)
		hostMenu->addAction(name);
	hostMenu->setCurrentItem(hosts.indexOf(host));

	hostConfig(host);
}

void DictWindow::hostConfig(const QString &host)
{
	QStringList list = host.split(':');
	if (list.size() > 1)
		dict->setHost(list[0], list[1].toInt());
	else
		dict->setHost(host, conf.port());

	conf.setHost(host);
	setHostStatus(false);

	QString username = conf.username();
	if (!username.isEmpty())
		dict->setUser(username, conf.password());

	DictEdit groups(conf.groups());
	dbmenu->setGroups(*groups.list());
	dbmenu->setChecked(conf.selected());
	if (conf.selected() == 2)
		tracker->setDatabase(conf.database());
	else
		tracker->setDatabase(dbmenu->current());
	tracker->setStrategy(conf.strategy());
}

void DictWindow::writeConfig()
{
	conf.writeSidebar(sidebar->currentIndex());
	conf.writeWidths(splitter->sizes());
	conf.writeFont(browser->textFont());

	conf.writeServer(hostMenu->items());
	if (databaseMenu->isEnabled())
		conf.writeHost(database(), strategy());
	conf.writeSelected(dbmenu->checked());

	conf.writeSelection(selection->conf());
	conf.write();
}

void DictWindow::setupActions()
{
	KAction *text = new KAction(this);
	text->setText(i18n("Page Source"));
	text->setCheckable(true);
	actionCollection()->addAction("text", text);

	connect(text, SIGNAL(toggled(bool)), browser, SLOT(plainText(bool)));

	KAction *format = new KAction(this);
	format->setText(i18n("Reformat"));
	actionCollection()->addAction("format", format);

	connect(format, SIGNAL(triggered()), SLOT(reformat()));

	yank = new KAction(this);
	yank->setText(i18n("Yank Selections"));
	yank->setCheckable(true);
	actionCollection()->addAction("yank", yank);

	connect(yank, SIGNAL(toggled(bool)), selection, SLOT(set(bool)));

	KAction *sidebar = new KAction(this);
	sidebar->setText(i18n("Database List"));
// 	sidebar->setCheckable(true);
	actionCollection()->addAction("sidebar", sidebar);

	connect(sidebar, SIGNAL(triggered(bool)), SLOT(toggleDatabaseList()));

	KAction *matches = new KAction(this);
	matches->setText(i18n("Match List"));
// 	matches->setCheckable(true);
	actionCollection()->addAction("matches", matches);

	connect(matches, SIGNAL(triggered()), SLOT(toggleMatchList()));

	KAction *define = new KAction(this);
	define->setText(i18n("Define"));
	define->setShortcut(Qt::CTRL + Qt::Key_D);
	actionCollection()->addAction("define", define);

	connect(define, SIGNAL(triggered()), SLOT(define()));

	KAction *match = new KAction(this);
	match->setText(i18n("Match"));
	match->setShortcut(Qt::CTRL + Qt::Key_M);
	actionCollection()->addAction("match", match);

	connect(match, SIGNAL(triggered()), SLOT(match()));

	KAction *clear = new KAction(this);
	clear->setText(i18n("Clear History"));
	actionCollection()->addAction("clear", clear);
// 	clear->setEnabled(false);

	connect(clear, SIGNAL(triggered()), browser, SLOT(clearHistory()));
	connect(clear, SIGNAL(triggered()), matchList, SLOT(clearHistory()));

	KAction *prev = new KAction(this);
	prev->setText(i18n("Previous Match"));
	prev->setShortcut(Qt::CTRL + Qt::Key_Left);
	actionCollection()->addAction("prev", prev);
	prev->setEnabled(false);

	connect(prev, SIGNAL(triggered()), matchList, SLOT(backward()));
	connect(matchList, SIGNAL(backwardAvailable(bool)),
		prev, SLOT(setEnabled(bool)));
	connect(prev, SIGNAL(triggered()), SLOT(openMatchList()));

	KAction *next = new KAction(this);
	next->setText(i18n("Next Match"));
	next->setShortcut(Qt::CTRL + Qt::Key_Right);
	actionCollection()->addAction("next", next);
	next->setEnabled(false);

	connect(next, SIGNAL(triggered()), matchList, SLOT(forward()));
	connect(matchList, SIGNAL(forwardAvailable(bool)),
		next, SLOT(setEnabled(bool)));
	connect(next, SIGNAL(triggered()), SLOT(openMatchList()));

	dictinfo = new DictInfo(this);
	dictinfo->setText(i18n("Info"));
	dictinfo->setIcon(KIcon("dialog-information"));
	actionCollection()->addAction("info", dictinfo);

	dictinfo->addAction(i18n("Server"), "server");
	dictinfo->addAction(i18n("Databases"), "databases");
	dictinfo->addAction(i18n("Strategies"), "strategies");
	dictinfo->addAction(i18n("Status"), "status");

	connect(dictinfo, SIGNAL(selected(const QString &)),
		SLOT(info(const QString &)));

	hostMenu = new KSelectAction(this);
	hostMenu->setText(i18n("Host"));
	hostMenu->setIcon(KIcon("network-server"));
	actionCollection()->addAction("host", hostMenu);

	connect(hostMenu, SIGNAL(triggered(const QString &)),
		SLOT(setHost(const QString &)));

	databaseMenu = new DictMenu(this);
	databaseMenu->setText(i18n("Database List"));
// 	actionCollection()->addAction("databaselist", databaseMenu);
	databaseMenu->setEnabled(false);

	connect(databaseMenu, SIGNAL(selected(const QStringList &)),
		databaseView, SLOT(setCurrent(const QStringList &)));
	connect(databaseView, SIGNAL(selected(const QStringList &)),
		databaseMenu, SLOT(setCurrent(const QStringList &)));

	edit = new KAction(this);
	edit->setText(i18n("Edit Databases..."));
	actionCollection()->addAction("edit", edit);
	edit->setEnabled(false);

	connect(edit, SIGNAL(triggered()), SLOT(setGroups()));

	dbmenu = new DictDatabase(this);
	dbmenu->setMenu(databaseMenu, edit);
	dbmenu->setText(i18n("Database"));
	dbmenu->setIcon(KIcon("network-server-database"));
	actionCollection()->addAction("database", dbmenu);

	connect(databaseView, SIGNAL(selected(const QStringList &)),
		dbmenu, SLOT(setList()));

	strategyMenu = new DictSelect(this);
	strategyMenu->setText(i18n("Strategy"));
	actionCollection()->addAction("strategy", strategyMenu);
	strategyMenu->setEnabled(false);

	KAction *backward = new KToolBarPopupAction(KIcon("go-previous"),
		i18n("Backward"), this);
	actionCollection()->addAction("backward", backward);
	backward->setShortcut(Qt::ALT + Qt::Key_Left);
	backward->setEnabled(false);

	connect(backward, SIGNAL(triggered()), browser, SLOT(backward()));
	connect(browser, SIGNAL(backwardAvailable(bool)),
		backward, SLOT(setEnabled(bool)));
	connect(backward, SIGNAL(triggered()), SLOT(cancel()));

	backwardMenu = new DictHistory(browser, DictHistory::Backward);
	backward->setMenu(backwardMenu);
	connect(backwardMenu, SIGNAL(triggered(QAction *)), SLOT(cancel()));

	KAction *forward = new KToolBarPopupAction(KIcon("go-next"),
		i18n("Forward"), this);
	actionCollection()->addAction("forward", forward);
	forward->setShortcut(Qt::ALT + Qt::Key_Right);
	forward->setEnabled(false);

	connect(forward, SIGNAL(triggered()), browser, SLOT(forward()));
	connect(browser, SIGNAL(forwardAvailable(bool)),
		forward, SLOT(setEnabled(bool)));

	connect(forward, SIGNAL(triggered()), SLOT(cancel()));

	forwardMenu = new DictHistory(browser, DictHistory::Forward);
	forward->setMenu(forwardMenu);
	connect(forwardMenu, SIGNAL(triggered(QAction *)), SLOT(cancel()));

	stop = new KAction(KIcon("process-stop"), i18n("Stop"), this);
	stop->setShortcut(Qt::Key_Escape);
	actionCollection()->addAction("stop", stop);
	stop->setEnabled(false);

	connect(tracker, SIGNAL(loading(bool)), stop, SLOT(setEnabled(bool)));
	connect(tracker, SIGNAL(loading(bool)), matchList, SLOT(busyCursor(bool)));
	connect(tracker, SIGNAL(loading(bool)), browser, SLOT(busyCursor(bool)));
	connect(stop, SIGNAL(triggered()), SLOT(cancel()));

	KAction *frame = new KAction(this);
	frame->setText(i18n("Next Frame"));
	actionCollection()->addAction("frame", frame);

	connect(frame, SIGNAL(triggered()), SLOT(nextFrame()));

	auth = new KAction(this);
	auth->setText(i18n("Authentication..."));
	auth->setIcon(KIcon("dialog-password"));
	actionCollection()->addAction("auth", auth);
	auth->setEnabled(false);

	connect(auth, SIGNAL(triggered()), SLOT(authenticate()));

	QLabel *label = new QLabel(this);
	label->setText(i18n("Look for:"));
	label->setIndent(10);
	KAction *action = new KAction(this);
	action->setDefaultWidget(label);
	action->setText(i18n("Look for"));
	actionCollection()->addAction("search", action);

	KAction *combo = new KAction(this);
	combo->setText(i18n("Query/Location Bar"));
	combo->setShortcut(Qt::CTRL + Qt::Key_L);
	combo->setDefaultWidget(entry);
	actionCollection()->addAction("query", combo);

	connect(combo, SIGNAL(triggered()), entry, SLOT(setFocus()));
	connect(combo, SIGNAL(triggered()), entry->lineEdit(), SLOT(selectAll()));
	connect(entry, SIGNAL(activated(int)), SLOT(define()));

	KAction *intro = new KAction(this);
	intro->setText(i18n("Introduction"));
	actionCollection()->addAction("intro", intro);

	connect(intro, SIGNAL(triggered()), SLOT(intro()));

	label = new QLabel(this);
	label->setText(i18n("Matching:"));
	label->setIndent(10);
	action = new KAction(this);
	action->setDefaultWidget(label);
	action->setText(i18n("Matching Strategy"));
	actionCollection()->addAction("matching", action);

	connect(action, SIGNAL(triggered()), strategyMenu, SLOT(setFocus()));

	action = KStandardAction::openNew(this, SLOT(newWindow()), actionCollection());
	action->setText(i18n("New Window"));
	action->setIcon(KIcon("window-new"));

	KStandardAction::saveAs(this, SLOT(save()), actionCollection());
	KStandardAction::print(this, SLOT(print()), actionCollection());
	KStandardAction::printPreview(this, SLOT(preview()), actionCollection());

	KStandardAction::close(this, SLOT(close()), actionCollection());
	KStandardAction::quit(kapp, SLOT(closeAllWindows()), actionCollection());

	KStandardAction::find(search, SLOT(show()), actionCollection());
	KStandardAction::findNext(search, SLOT(next()), actionCollection());
	KStandardAction::findPrev(search, SLOT(prev()), actionCollection());

	KStandardAction::showMenubar(this, SLOT(toggleMenubar()), actionCollection());
	KStandardAction::preferences(this, SLOT(configure()), actionCollection());

	connect(selection, SIGNAL(selectionChanged(const QString &)),
		SLOT(yankClipboard(const QString &)));

	KStandardAction::help(this, SLOT(help()), actionCollection());

	KHelpMenu *helpmenu = new KHelpMenu(this, KGlobal::mainComponent().aboutData());
	actionCollection()->addAction(KStandardAction::SwitchApplicationLanguage, helpmenu, SLOT(switchApplicationLanguage()));
	KStandardAction::aboutApp(helpmenu, SLOT(aboutApplication()), actionCollection());
	KStandardAction::aboutKDE(helpmenu, SLOT(aboutKDE()), actionCollection());

	setHelpMenuEnabled(false);
}

void DictWindow::setupBrowser()
{
	connect(browser, SIGNAL(historyChanged(const QString &)),
		entry, SLOT(setEditText(const QString &)));
	connect(browser, SIGNAL(historyChanged(const QString &)),
		SLOT(setCaption(const QString &)));

	connect(browser, SIGNAL(anchorClicked(const QUrl &)),
		SLOT(define(const QUrl &)));
	connect(browser, SIGNAL(highlighted(const QString &)),
		statusBar(), SLOT(showMessage(const QString &)));

	connect(browser, SIGNAL(historyChanged(const QString &)),
		SLOT(setQuery(const QString &)));
	connect(browser, SIGNAL(historyChanged(const QString &)),
		search, SLOT(clear()));

	connect(matchList, SIGNAL(anchorClicked(const QUrl &)),
		SLOT(define(const QUrl &)));
	connect(matchList, SIGNAL(highlighted(const QString &)),
		statusBar(), SLOT(showMessage(const QString &)));
}

void DictWindow::statusMessage(const QString &message)
{
	statusBar()->showMessage(message, (message.endsWith("...")) ? 0 : 4000);
}

void DictWindow::setHostStatus(bool connected)
{
	if (!connected) cancel();
	hostStatus->setStatus("network-workgroup", dicthost(), connected);
	hostStatus->setTip((connected) ? i18n("Connected") : i18n("Disconnected"));
}

void DictWindow::setUserStatus(bool authenticated)
{
	if (!userStatus)
		userStatus = new DictStatus(statusBar());
	if (authenticated) {
		if (userStatus->isVisible())
			return;
		userStatus->setStatus("user-identity", dict->user(), authenticated);
		userStatus->setTip(i18n("User"));
		statusBar()->addPermanentWidget(userStatus);
		statusBar()->addPermanentWidget(hostStatus);
 		userStatus->show();
	} else
		statusBar()->removeWidget(userStatus);
}

void DictWindow::setCapabilities(bool identified)
{
	auth->setEnabled((identified && dict->capabilities().contains("auth"))
		? true : false);
}

void DictWindow::setupStatusbar()
{
	progress = new DictProgress(statusBar());
	statusBar()->addPermanentWidget(progress);
	connect(tracker, SIGNAL(progress(int, int)), progress, SLOT(set(int, int)));

	hostStatus = new DictStatus(statusBar());
	statusBar()->addPermanentWidget(hostStatus);
	userStatus = 0;
}

void DictWindow::setupDict()
{
	connect(dict, SIGNAL(statusMessage(const QString &)),
		SLOT(statusMessage(const QString &)));
	connect(dict, SIGNAL(errorMessage(const QString &, int)),
		SLOT(error(const QString &, int)));

	connect(dict, SIGNAL(responseProcessed(const DictResponse &)),
		tracker, SLOT(response(const DictResponse &)));
	connect(dict, SIGNAL(connected(bool)), SLOT(setHostStatus(bool)));
	connect(dict, SIGNAL(authenticated(bool)), SLOT(setUserStatus(bool)));
	connect(dict, SIGNAL(identified(bool)), SLOT(setCapabilities(bool)));

	dict->setClient(KProtocolManager::userAgentForApplication(Dikt::AppName, Dikt::Version));
	dict->setTimeouts(KProtocolManager::connectTimeout(), KProtocolManager::readTimeout());
}

void DictWindow::setupTracker()
{
	connect(tracker, SIGNAL(databases(const DictList &)),
		SLOT(setDatabaseMenu(const DictList &)));
	connect(tracker, SIGNAL(strategies(const DictList &)),
		SLOT(setStrategyMenu(const DictList &)));
	connect(tracker, SIGNAL(matches(bool)), SLOT(openMatchList(bool)));

	connect(databaseView, SIGNAL(selected(const QStringList &)),
		tracker, SLOT(setDatabase(const QStringList &)));
	connect(databaseMenu, SIGNAL(selected(const QStringList &)),
		tracker, SLOT(setDatabase(const QStringList &)));
	connect(dbmenu, SIGNAL(selected(const QStringList &)),
		tracker, SLOT(setDatabase(const QStringList &)));
	connect(strategyMenu, SIGNAL(triggered(const QString &)),
		tracker, SLOT(setStrategy(const QString &)));
}

void DictWindow::setDatabaseMenu(const DictList &list)
{
	QStringList selected = ((databaseMenu->isEnabled())
		? database() : conf.database());
	databaseMenu->setList(list, selected);
	databaseView->setList(list, selected);
	dbmenu->checkList();
	tracker->setDatabase(dbmenu->current());

	edit->setEnabled(true);
}

void DictWindow::setStrategyMenu(const DictList &list)
{
	QString selected = strategy();
	if (selected.isEmpty())
		selected = conf.strategy();
	strategyMenu->setList(list, selected);
}

void DictWindow::setGroups()
{
	DictEdit groups(conf.groups());
	DictEditConf groupconf(this);
	groupconf.set(&groups, databaseView->list());
	if (groupconf.exec()) {
		groups.writeConf();
		conf.writeGroups(groups.groups());
		dbmenu->setGroups(*groups.list());
		tracker->setDatabase(dbmenu->current());
	}
}

void DictWindow::define(const QString &text)
{
	if (text.isEmpty()) return;
	query = text.simplified();
	query.truncate(maxQueryLen);

	if (stop->isEnabled())
		return;
	browser->setFocus();

	QUrl url(query);
	if (url.isRelative())
		tracker->define(query);
	else if (url.scheme() == "about")
		about(query);
	else if (url.scheme() == "dict")
		dicturl(url);
	else if (url.scheme() == "http")
		warning(text, DictPage::UrlRequested);
	else if (url.host().isEmpty())
		tracker->define(query);
	else
		error(i18n("Unsupported protocol %1", url.scheme()), DictPage::UnsupportedScheme);
}

void DictWindow::define()
{
	define(entry->currentText());
}

void DictWindow::define(const QUrl &link)
{
	if (link.scheme() == "http")
		KToolInvocation::invokeBrowser(link.toString());
	else
		define(link.toString());
}

void DictWindow::match(const QString &text)
{
	if (text.isEmpty()) return;
	query = text.simplified();
	query.truncate(maxQueryLen);

	if (stop->isEnabled())
		return;
	browser->setFocus();
	tracker->match(query, strategy());
}

void DictWindow::match()
{
	match(entry->currentText());
}

void DictWindow::about(const QString &query)
{
	QString text = query.mid(6);
	if (text.isEmpty())
		page->about(query, Dikt::Version);
	else if (!text.compare(Dikt::AppName, Qt::CaseInsensitive)) intro();
	else if (!text.compare("help", Qt::CaseInsensitive)) page->help(query);
	else if (dictinfo->list().contains(text))
		info(query, text);
	else if (databaseMenu->list().contains(text))
		info(query, "info " + text);
	else error(i18n("No such database %1", text), DictPage::UnknownDatabase);
}

void DictWindow::info(const QString &query, const QString &text)
{
	if (stop->isEnabled())
		return;
	browser->setFocus();
	tracker->info(query, text);
}

void DictWindow::warning(const QString &text, int code)
{
	page->setTitle(text.simplified());
	page->warning(text, (DictPage::Warning)code);
	browser->setFocus();
}

void DictWindow::error(const QString &text, int code)
{
	page->setTitle(query);
	page->error(text, code);
	browser->setFocus();
	tracker->cancel();
}

void DictWindow::cancel()
{
	if (stop->isEnabled())
		tracker->cancel();
	progress->hide();
}

void DictWindow::dicturl(const QUrl &url)
{
	QString host = url.host();
	if (host.isEmpty()) {
		error(i18n("Malformed Url"), DictPage::MalformedURL);
		return;
	}
	int port = url.port(Dict::DefaultPort);
	if (port != Dict::DefaultPort)
		host.append(':').append(QString::number(port));

	if (host != dicthost()) {
		int index = hostMenu->items().indexOf(host);
		if (index < 0) {
			index = hostMenu->items().size();
			hostMenu->addAction(host);
		}
		hostMenu->setCurrentItem(index);
		setHost(host);
	}
	QString path = url.path();
	if (path.size() <= 1)
		info(url.toString(), "status");
	else
		define(path.mid(1));
}

void DictWindow::setHost(const QString &host)
{
	cancel();
	if (databaseMenu->isEnabled())
		conf.writeHost(database(), strategy());
	conf.writeSelected(dbmenu->checked());
	databaseMenu->clear();
	databaseView->clear();
	strategyMenu->clear();
	databaseMenu->setEnabled(false);
	strategyMenu->setEnabled(false);
	dbmenu->setEnabled(false);
	edit->setEnabled(false);
	hostConfig(host);
	tracker->init();
	if (userStatus)
		setUserStatus(false);
	setCapabilities(false);
	page->dictformatter()->clear();
}

void DictWindow::yankClipboard(const QString &text)
{
	if (QUrl(text.simplified()).scheme() == "http")
		warning(text, DictPage::UrlSelection);
	else if (text.length() > selection->conf().warnLen)
		warning(text, DictPage::LongSelection);
	else
		define(text);
}

void DictWindow::authenticate()
{
	KPasswordDialog auth(this, KPasswordDialog::ShowUsernameLine | KPasswordDialog::ShowKeepPassword);

	auth.setCaption(i18n("Dict Authentication"));
	auth.setPrompt(i18n("Enter your authentication for host <b>%1</b>", dict->host()));
	auth.setUsername(dict->user());

	if (!auth.exec())
		return;
	dict->auth(auth.username(), auth.password());

	if (auth.keepPassword())
		conf.writeUser(auth.username(), auth.password());
	else
		conf.deleteUser();
}

void DictWindow::toggleMenubar()
{
	menuBar()->setVisible(!menuBar()->isVisible());
}

void DictWindow::setSidebar(QWidget *widget)
{
	conf.writeWidths(splitter->sizes());

	sidebar->setVisible(!widget->isVisible());
	sidebar->setCurrentWidget(widget);

	if (sidebar->isVisible())
		widget->setFocus();
	else
		browser->setFocus();
}

void DictWindow::setSidebar(int side)
{
	splitter->insertWidget(side, sidebar);
}

void DictWindow::toggleDatabaseList()
{
	setSidebar(databaseView);
}

void DictWindow::toggleMatchList()
{
	setSidebar(matchList);
}

void DictWindow::openMatchList(bool focus)
{
	if(!matchList->isVisible()) {
		sidebar->show();
		sidebar->setCurrentWidget(matchList);
	}
	if (focus) matchList->setFocus();
}

void DictWindow::enableMatchList(bool enable)
{
	actionCollection()->action("matches")->setEnabled(enable);
	if (!enable) {
		actionCollection()->action("prev")->setEnabled(enable);
		actionCollection()->action("next")->setEnabled(enable);
		if (matchList->isVisible())
			sidebar->hide();
		matchList->clearHistory();
	}
}

void DictWindow::nextFrame()
{
	if (browser->hasFocus())
		if (sidebar->isVisible())
			sidebar->currentWidget()->setFocus();
		else if (search->isVisible())
			search->setFocus();
		else
			entry->setFocus();
	else
		browser->setFocus();
}

void DictWindow::configure()
{
	if (!settings) {
		settings = new DictSettings(this);
		connect(settings, SIGNAL(accepted()), SLOT(loadSettings()));
		connect(settings, SIGNAL(applyClicked()), SLOT(loadSettings()));
	}
	settings->setHosts(hostMenu->items());
	settings->setFont(browser->textFont());
	settings->setStyle(style);
	settings->setSelection(selection->conf());
	settings->readConf(conf);
	settings->show();
}

void DictWindow::loadSettings()
{
	QStringList hosts = settings->hosts();
	if (hosts.isEmpty())
		hosts << Dict::DefaultHost;
	QString host = dicthost();
	int index = qMax(hosts.indexOf(host), 0);
	hostMenu->setItems(hosts);
	hostMenu->setCurrentItem(index);
	if (host != dicthost())
		setHost(dicthost());

	browser->setTextFont(settings->font());
	matchList->setTextFont(settings->font());
	style->writeConf();
	selection->setup(settings->selection());
	settings->writeConf(conf);
	loadConfig();
}

void DictWindow::loadConfig()
{
	browser->setStyleSheet(style->stylesheet());
	matchList->setStyleSheet(style->stylesheet());
	page->setReformat(conf.reformat);
	page->setHeadings(conf.headings);
	matchList->dictPage()->setHeadings(conf.headings);

	setSidebar(conf.sidebarSide);
	enableMatchList(conf.matchPage);
	tracker->setDefaultDatabase(conf.defaultSearch);
	tracker->setCorrect(conf.corrections);
	tracker->setAutoMatch(conf.autoMatch);
	tracker->setAutoStrategy(conf.autoStrategy);
	tracker->setMatchPage((conf.matchPage) ? matchList->dictPage() : 0);

	yank->setChecked(selection->conf().yank);
}

void DictWindow::save()
{
	static QString docPath = KGlobalSettings::documentPath();
	QString fileName = QFileDialog::getSaveFileName(this, i18n("Save Document"), docPath + "/" + entry->currentText() + ".html");
	if (fileName.isEmpty())
		return;
	docPath = fileName.left(fileName.lastIndexOf('/'));

	QFile file(fileName);
	if (file.open(QFile::WriteOnly | QFile::Text)) {
		QTextStream out(&file);
		out.setCodec("UTF-8");
		out << browser->pageText();
		statusMessage(i18n("Document saved"));
	} else
		error(i18n("Cannot write file <b>%1</b>: %2").arg(fileName, file.errorString()), DictPage::FileSystem);
}

void DictWindow::print()
{
	QPrinter printer;
	if (browser->textCursor().hasSelection())
		printer.setPrintRange(QPrinter::Selection);
	QPrintDialog dialog(&printer, this);
	dialog.setWindowTitle(i18n("Print Document"));
	if (dialog.exec() == QDialog::Accepted)
		browser->print(&printer);
}

void DictWindow::preview()
{
	QPrinter printer;
	if (browser->textCursor().hasSelection())
		printer.setPrintRange(QPrinter::Selection);
	QPrintPreviewDialog dialog(&printer, this);
	connect(&dialog, SIGNAL(paintRequested(QPrinter *)),
		browser, SLOT(preview(QPrinter *)));
	dialog.exec();
}

void DictWindow::reformat()
{
	page->dictformatter()->readConf();
	browser->setStyleSheet(style->stylesheet());
	define(query);
}
